/*!
 * Contro
 * (c) 2017 Niklas Higi
 * Released under the MIT License.
 */
(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
  typeof define === 'function' && define.amd ? define(['exports'], factory) :
  (factory((global.Contro = {})));
}(this, (function (exports) { 'use strict';

  class Vector2 {
      constructor(x = 0, y = 0) {
          this.x = x;
          this.y = y;
      }
  }

  (function (MouseButton) {
      MouseButton[MouseButton["Left"] = 0] = "Left";
      MouseButton[MouseButton["Middle"] = 1] = "Middle";
      MouseButton[MouseButton["Right"] = 2] = "Right";
  })(exports.MouseButton || (exports.MouseButton = {}));
  class Mouse {
      constructor({ canvas, doc = document }) {
          this.pointerLocked = false;
          this.pointerMovement = new Vector2();
          this.pressedButtons = new Set();
          this.queuedButtons = new Set();
          this.scrollDistance = 0;
          this.canvas = canvas;
          this.document = doc;
          let on = this.canvas.addEventListener.bind(this.canvas);
          on('mousedown', (event) => {
              this.pressedButtons.add(event.button);
              this.queuedButtons.add(event.button);
          });
          on('mouseup', (event) => {
              this.pressedButtons.delete(event.button);
              this.queuedButtons.delete(event.button);
          });
          on('mousemove', (event) => {
              this.pointerMovement.x += event.movementX;
              this.pointerMovement.y += event.movementY;
          });
          on('wheel', (event) => {
              const distance = event.deltaY;
              this.scrollDistance += distance;
          });
          on = this.document.addEventListener.bind(this.document);
      }
      isPressed(button = exports.MouseButton.Left) {
          return this.pressedButtons.has(button);
      }
      wasPressed(button = exports.MouseButton.Left) {
          if (this.queuedButtons.has(button)) {
              this.queuedButtons.delete(button);
              return true;
          }
          return false;
      }
      getPointerMovement() {
          const movement = this.pointerMovement;
          this.pointerMovement = new Vector2(0, 0);
          return movement;
      }
      getScrollDistance() {
          const distance = this.scrollDistance;
          this.scrollDistance = 0;
          return distance;
      }
      lockPointer() {
          this.canvas.requestPointerLock();
      }
      unlockPointer() {
          this.document.exitPointerLock();
      }
      isPointerLocked() {
          return this.document.pointerLockElement === this.canvas;
      }
  }

  const arrowKeyTemplates = {
      arrows: ['ArrowUp', 'ArrowLeft', 'ArrowDown', 'ArrowRight'],
      wasd: ['W', 'A', 'S', 'D'],
  };
  class Keyboard {
      constructor(
          { doc = document } = {}) {
          this.pressedKeys = new Set();
          this.queuedKeys = new Set();
          this.document = doc;
          this.document.addEventListener('keydown', (event) => {
              const key = event.key.toLowerCase();
              this.pressedKeys.add(key);
              this.queuedKeys.add(key);
          });
          this.document.addEventListener('keyup', (event) => {
              const key = event.key.toLowerCase();
              this.pressedKeys.delete(key);
              this.queuedKeys.delete(key);
          });
      }
      isPressed(key) {
          key = key.toLowerCase();
          return this.pressedKeys.has(key);
      }
      wasPressed(key) {
          key = key.toLowerCase();
          if (this.queuedKeys.has(key)) {
              this.queuedKeys.delete(key);
              return true;
          }
          return false;
      }
      getMovementVector(arrowKeys) {
          if (typeof arrowKeys === 'string') {
              arrowKeys = arrowKeys.toLowerCase();
              if (arrowKeys in arrowKeyTemplates) {
                  arrowKeys = arrowKeyTemplates[arrowKeys];
              }
              else {
                  throw new Error(`Arrow key template "${arrowKeys}" not found!`);
              }
          }
          const vector = new Vector2();
          if (this.isPressed(arrowKeys[0]))
              vector.y -= 1;
          if (this.isPressed(arrowKeys[1]))
              vector.x -= 1;
          if (this.isPressed(arrowKeys[2]))
              vector.y += 1;
          if (this.isPressed(arrowKeys[3]))
              vector.x += 1;
          return vector;
      }
  }

  class Gamepad {
      constructor(
          { win = window, nav = navigator } = {}) {
          this.pressedButtons = new Set();
          this.window = win;
          this.navigator = nav;
          this.window.addEventListener('gamepadconnected', ({ gamepad }) => {
              if (!this.isConnected())
                  this.gamepadIndex = gamepad.index;
          });
          this.window.addEventListener('gamepaddisconnected', ({ gamepad }) => {
              if (this.gamepadIndex === gamepad.index) {
                  const gamepads = this.navigator.getGamepads();
                  if (gamepads.length) {
                      this.gamepadIndex = this.navigator.getGamepads()[0].index;
                  }
                  else {
                      this.gamepadIndex = undefined;
                  }
              }
          });
      }
      isConnected() {
          return this.gamepadIndex !== undefined && this.gamepad.connected;
      }
      get gamepad() {
          return this.navigator.getGamepads()[this.gamepadIndex];
      }
      isPressed(button) {
          return this.isConnected() && this.gamepad.buttons[button].pressed;
      }
      wasPressed(button) {
          if (this.isConnected()) {
              if (this.gamepad.buttons[button].pressed) {
                  if (!this.pressedButtons.has(button)) {
                      this.pressedButtons.add(button);
                      return true;
                  }
              }
              else {
                  this.pressedButtons.delete(button);
              }
          }
          return false;
      }
  }

  exports.Mouse = Mouse;
  exports.Keyboard = Keyboard;
  exports.Gamepad = Gamepad;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
